@@ -30,7 +30,7 @@ where
     core_data_reader: BitReader<CDR>,
     external_data_readers: HashMap<Itf8, EDR>,
     reference_sequence_id: ReferenceSequenceId,
-    prev_alignment_start: Itf8,
+    prev_alignment_start: Option<sam::record::Position>,
 }
 
 impl<'a, CDR, EDR> Reader<'a, CDR, EDR>
@@ -43,7 +43,7 @@ where
         core_data_reader: BitReader<CDR>,
         external_data_readers: HashMap<Itf8, EDR>,
         reference_sequence_id: ReferenceSequenceId,
-        initial_alignment_start: Itf8,
+        initial_alignment_start: Option<sam::record::Position>,
     ) -> Self {
         Self {
             compression_header,
@@ -135,8 +135,9 @@ where
         let read_length = self.read_read_length().await?;
         builder = builder.set_read_length(read_length);
 
-        let alignment_start = self.read_alignment_start().await?;
-        builder = builder.set_alignment_start(alignment_start);
+        if let Some(alignment_start) = self.read_alignment_start().await? {
+            builder = builder.set_alignment_start(alignment_start);
+        }
 
         let read_group = self.read_read_group().await?;
         builder = builder.set_read_group_id(read_group);
@@ -179,7 +180,7 @@ where
         .and_then(|n| usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e)))
     }
 
-    async fn read_alignment_start(&mut self) -> io::Result<Itf8> {
+    async fn read_alignment_start(&mut self) -> io::Result<Option<sam::record::Position>> {
         let ap_data_series_delta = self
             .compression_header
             .preservation_map()
@@ -190,7 +191,7 @@ where
             .data_series_encoding_map()
             .in_seq_positions_encoding();
 
-        let alignment_start = decode_itf8(
+        let mut alignment_start = decode_itf8(
             encoding,
             &mut self.core_data_reader,
             &mut self.external_data_readers,
@@ -198,9 +199,17 @@ where
         .await?;
 
         if ap_data_series_delta {
-            Ok(self.prev_alignment_start + alignment_start)
+            let prev_alignment_start = self.prev_alignment_start.map(i32::from).unwrap_or_default();
+            let delta = alignment_start;
+            alignment_start = prev_alignment_start + delta;
+        }
+
+        if alignment_start == 0 {
+            Ok(None)
         } else {
-            Ok(alignment_start)
+            sam::record::Position::try_from(alignment_start)
+                .map(Some)
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
         }
     }
 
@@ -281,8 +290,9 @@ where
                 builder = builder.set_next_fragment_reference_sequence_id(id);
             }
 
-            let next_mate_alignment_start = self.read_next_mate_alignment_start().await?;
-            builder = builder.set_next_mate_alignment_start(next_mate_alignment_start);
+            if let Some(next_mate_alignment_start) = self.read_next_mate_alignment_start().await? {
+                builder = builder.set_next_mate_alignment_start(next_mate_alignment_start);
+            }
 
             let template_size = self.read_template_size().await?;
             builder = builder.set_template_size(template_size);
@@ -349,7 +359,9 @@ where
         })
     }
 
-    async fn read_next_mate_alignment_start(&mut self) -> io::Result<Itf8> {
+    async fn read_next_mate_alignment_start(
+        &mut self,
+    ) -> io::Result<Option<sam::record::Position>> {
         let encoding = self
             .compression_header
             .data_series_encoding_map()
@@ -367,6 +379,15 @@ where
             &mut self.external_data_readers,
         )
         .await
+        .and_then(|n| {
+            if n == 0 {
+                Ok(None)
+            } else {
+                sam::record::Position::try_from(n)
+                    .map(Some)
+                    .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+            }
+        })
     }
 
     async fn read_template_size(&mut self) -> io::Result<Itf8> {
@@ -410,6 +431,8 @@ where
     }
 
     async fn read_tag_data(&mut self) -> io::Result<Vec<Tag>> {
+        use bam::reader::record::data::field::read_value;
+
         let tag_line = self.read_tag_line().await.and_then(|i| {
             usize::try_from(i).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
         })?;
@@ -441,8 +464,8 @@ where
             )
             .await?;
 
-            let mut data_reader = bam::record::data::Reader::new(&data[..]);
-            let value = data_reader.read_value_type(key.ty())?;
+            let mut data_reader = &data[..];
+            let value = read_value(&mut data_reader, key.ty())?;
 
             let tag = Tag::new(*key, value);
             tags.push(tag);
@@ -907,7 +930,7 @@ where
 
             reader.read_u8().await
         }
-        _ => todo!("decode_byte: {:?}", encoding),
+        _ => unimplemented!("unsupported u8 encoding: {:?}", encoding),
     }
 }
 
@@ -933,13 +956,13 @@ where
 
             read_itf8(reader).await
         }
-        _ => todo!("decode_itf8: {:?}", encoding),
+        _ => unimplemented!("unsupported ITF8 encoding: {:?}", encoding),
     }
 }
 
 async fn decode_byte_array<CDR, EDR>(
     encoding: &Encoding,
-    _core_data_reader: &mut BitReader<CDR>,
+    core_data_reader: &mut BitReader<CDR>,
     external_data_readers: &mut HashMap<Itf8, EDR>,
 ) -> io::Result<Vec<u8>>
 where
@@ -947,6 +970,32 @@ where
     EDR: AsyncBufRead + Unpin,
 {
     match encoding {
+        Encoding::ByteArrayLen(len_encoding, value_encoding) => {
+            let len = decode_itf8(len_encoding, core_data_reader, external_data_readers)
+                .await
+                .and_then(|n| {
+                    usize::try_from(n).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+                })?;
+
+            let mut buf = vec![0; len];
+
+            if let Encoding::External(block_content_id) = **value_encoding {
+                let reader = external_data_readers
+                    .get_mut(&block_content_id)
+                    .ok_or_else(|| {
+                        io::Error::new(
+                            io::ErrorKind::InvalidData,
+                            ReadRecordError::MissingExternalBlock(block_content_id),
+                        )
+                    })?;
+
+                reader.read_exact(&mut buf).await?;
+
+                Ok(buf)
+            } else {
+                unimplemented!("unsupported [u8] encoding: {:?}", encoding);
+            }
+        }
         Encoding::ByteArrayStop(stop_byte, block_content_id) => {
             let reader = external_data_readers
                 .get_mut(block_content_id)
@@ -965,7 +1014,7 @@ where
 
             Ok(buf)
         }
-        _ => todo!("decode_byte_array: {:?}", encoding),
+        _ => unimplemented!("unsupported [u8] encoding: {:?}", encoding),
     }
 }
 
@@ -994,4 +1043,62 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_decode_itf8() -> io::Result<()> {
+        async fn t(encoding: &Encoding, expected: Itf8) -> io::Result<()> {
+            let core_data = [0b10000000];
+            let mut core_data_reader = BitReader::new(&core_data[..]);
+
+            let external_data = [0x0d];
+            let mut external_data_readers = vec![(1, &external_data[..])].into_iter().collect();
+
+            let actual =
+                decode_itf8(encoding, &mut core_data_reader, &mut external_data_readers).await?;
+
+            assert_eq!(expected, actual);
+
+            Ok(())
+        }
+
+        t(&Encoding::External(1), 13).await?;
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_decode_byte_array() -> io::Result<()> {
+        async fn t(external_data: &[u8], encoding: &Encoding, expected: &[u8]) -> io::Result<()> {
+            let core_data = [];
+            let mut core_data_reader = BitReader::new(&core_data[..]);
+
+            let mut external_data_readers = vec![(1, external_data)].into_iter().collect();
+
+            let actual =
+                decode_byte_array(encoding, &mut core_data_reader, &mut external_data_readers)
+                    .await?;
+
+            assert_eq!(expected, actual);
+
+            Ok(())
+        }
+
+        let len_encoding = Encoding::External(1);
+        let value_encoding = Encoding::External(1);
+        t(
+            &[0x04, 0x6e, 0x64, 0x6c, 0x73],
+            &Encoding::ByteArrayLen(Box::new(len_encoding), Box::new(value_encoding)),
+            b"ndls",
+        )
+        .await?;
+
+        t(
+            &[0x6e, 0x64, 0x6c, 0x73, 0x00],
+            &Encoding::ByteArrayStop(0x00, 1),
+            b"ndls",
+        )
+        .await?;
+
+        Ok(())
+    }
 }
