@@ -18,7 +18,7 @@ use std::{
 use byteorder::{LittleEndian, ReadBytesExt};
 
 use self::container::read_container;
-use super::{container::Block, file_definition::Version, Container, FileDefinition, MAGIC_NUMBER};
+use super::{container::Block, file_definition::Version, FileDefinition, MAGIC_NUMBER};
 
 /// A CRAM reader.
 ///
@@ -67,6 +67,54 @@ where
         Self { inner: reader }
     }
 
+    /// Returns a reference to the underlying reader.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use std::io;
+    /// use noodles_cram as cram;
+    /// let data = [];
+    /// let reader = cram::Reader::new(&data[..]);
+    /// assert!(reader.get_ref().is_empty());
+    /// # Ok::<(), io::Error>(())
+    /// ```
+    pub fn get_ref(&self) -> &R {
+        &self.inner
+    }
+
+    /// Returns a mutable reference to the underlying reader.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use std::io;
+    /// use noodles_cram as cram;
+    /// let data = [];
+    /// let mut reader = cram::Reader::new(&data[..]);
+    /// assert!(reader.get_mut().is_empty());
+    /// # Ok::<(), io::Error>(())
+    /// ```
+    pub fn get_mut(&mut self) -> &mut R {
+        &mut self.inner
+    }
+
+    /// Unwraps and returns the underlying reader.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use std::io;
+    /// use noodles_cram as cram;
+    /// let data = [];
+    /// let reader = cram::Reader::new(&data[..]);
+    /// assert!(reader.into_inner().is_empty());
+    /// # Ok::<(), io::Error>(())
+    /// ```
+    pub fn into_inner(self) -> R {
+        self.inner
+    }
+
     /// Reads the CRAM file definition.
     ///
     /// The CRAM magic number is also checked.
@@ -124,8 +172,11 @@ where
         }
     }
 
-    pub(crate) fn read_container(&mut self) -> io::Result<Container> {
-        read_container(&mut self.inner)
+    pub(crate) fn read_data_container_with_container_header(
+        &mut self,
+    ) -> io::Result<Option<(crate::container::Header, DataContainer)>> {
+        use self::data_container::read_data_container_with_container_header;
+        read_data_container_with_container_header(&mut self.inner)
     }
 
     /// Reads a data container.
