@@ -919,8 +919,12 @@ where
             reader.read_u8()
         }
         Encoding::Huffman(alphabet, bit_lens) => {
-            let decoder = CanonicalHuffmanDecoder::new(alphabet, bit_lens);
-            decoder.decode(core_data_reader).map(|i| i as u8)
+            if alphabet.len() == 1 {
+                Ok(alphabet[0] as u8)
+            } else {
+                let decoder = CanonicalHuffmanDecoder::new(alphabet, bit_lens);
+                decoder.decode(core_data_reader).map(|i| i as u8)
+            }
         }
         Encoding::Beta(offset, len) => core_data_reader
             .read_u32(*len)
@@ -952,8 +956,12 @@ where
             read_itf8(reader)
         }
         Encoding::Huffman(alphabet, bit_lens) => {
-            let decoder = CanonicalHuffmanDecoder::new(alphabet, bit_lens);
-            decoder.decode(core_data_reader)
+            if alphabet.len() == 1 {
+                Ok(alphabet[0])
+            } else {
+                let decoder = CanonicalHuffmanDecoder::new(alphabet, bit_lens);
+                decoder.decode(core_data_reader)
+            }
         }
         Encoding::Beta(offset, len) => core_data_reader.read_u32(*len).map(|i| (i as i32 - offset)),
         _ => todo!("decode_itf8: {:?}", encoding),
@@ -1020,3 +1028,94 @@ where
         _ => todo!("decode_byte_array: {:?}", encoding),
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_decode_byte() -> io::Result<()> {
+        fn t(encoding: &Encoding, expected: u8) -> io::Result<()> {
+            let core_data = [0b10000000];
+            let mut core_data_reader = BitReader::new(&core_data[..]);
+
+            let external_data = [0x0d];
+            let mut external_data_readers = ExternalDataReaders::new();
+            external_data_readers.insert(1, &external_data[..]);
+
+            let actual = decode_byte(encoding, &mut core_data_reader, &mut external_data_readers)?;
+
+            assert_eq!(expected, actual);
+
+            Ok(())
+        }
+
+        t(&Encoding::External(1), 0x0d)?;
+        t(&Encoding::Huffman(vec![0x4e], vec![0]), 0x4e)?;
+        t(&Encoding::Beta(1, 3), 3)?;
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_decode_itf8() -> io::Result<()> {
+        fn t(encoding: &Encoding, expected: Itf8) -> io::Result<()> {
+            let core_data = [0b10000000];
+            let mut core_data_reader = BitReader::new(&core_data[..]);
+
+            let external_data = [0x0d];
+            let mut external_data_readers = ExternalDataReaders::new();
+            external_data_readers.insert(1, &external_data[..]);
+
+            let actual = decode_itf8(encoding, &mut core_data_reader, &mut external_data_readers)?;
+
+            assert_eq!(expected, actual);
+
+            Ok(())
+        }
+
+        t(&Encoding::External(1), 13)?;
+        t(&Encoding::Huffman(vec![0x4e], vec![0]), 0x4e)?;
+        t(&Encoding::Beta(1, 3), 3)?;
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_decode_byte_array() -> io::Result<()> {
+        fn t(external_data: &[u8], encoding: &Encoding, expected: &[u8]) -> io::Result<()> {
+            let core_data = [];
+            let mut core_data_reader = BitReader::new(&core_data[..]);
+
+            let mut external_data_readers = ExternalDataReaders::new();
+            external_data_readers.insert(1, external_data);
+
+            let actual = decode_byte_array(
+                encoding,
+                &mut core_data_reader,
+                &mut external_data_readers,
+                None,
+            )?;
+
+            assert_eq!(expected, actual);
+
+            Ok(())
+        }
+
+        let len_encoding = Encoding::External(1);
+        let value_encoding = Encoding::External(1);
+        t(
+            &[0x04, 0x6e, 0x64, 0x6c, 0x73],
+            &Encoding::ByteArrayLen(Box::new(len_encoding), Box::new(value_encoding)),
+            b"ndls",
+        )?;
+
+        t(
+            &[0x6e, 0x64, 0x6c, 0x73, 0x00],
+            &Encoding::ByteArrayStop(0x00, 1),
+            b"ndls",
+        )?;
+
+        Ok(())
+    }
+}
