@@ -1,6 +1,7 @@
 //! CRAM record and fields.
 
 mod builder;
+mod convert;
 pub mod feature;
 mod flags;
 mod next_mate_flags;
@@ -26,12 +27,12 @@ pub struct Record {
     pub(crate) cram_bit_flags: Flags,
     pub(crate) reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
     pub(crate) read_length: usize,
-    pub(crate) alignment_start: i32,
+    pub(crate) alignment_start: Option<sam::record::Position>,
     pub(crate) read_group: ReadGroupId,
     pub(crate) read_name: Vec<u8>,
     pub(crate) next_mate_bit_flags: NextMateFlags,
     pub(crate) next_fragment_reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
-    pub(crate) next_mate_alignment_start: i32,
+    pub(crate) next_mate_alignment_start: Option<sam::record::Position>,
     pub(crate) template_size: i32,
     pub(crate) distance_to_next_fragment: i32,
     pub(crate) tags: Vec<Tag>,
@@ -79,37 +80,21 @@ impl Record {
     }
 
     /// Returns the alignment start position.
-    pub fn alignment_start(&self) -> i32 {
+    ///
+    /// This value is 1-based.
+    pub fn alignment_start(&self) -> Option<sam::record::Position> {
         self.alignment_start
     }
 
     /// Returns the alignment end position.
+    ///
+    /// This value is 1-based.
     pub fn alignment_end(&self) -> i32 {
-        let mut alignment_span = self.read_length() as i32;
-
-        // - if does not consume reference; + if does not consume read
-        for feature in self.features() {
-            match feature {
-                Feature::Insertion(_, bases) => {
-                    alignment_span -= bases.len() as i32;
-                }
-                Feature::InsertBase(_, _) => {
-                    alignment_span -= 1;
-                }
-                Feature::Deletion(_, len) => {
-                    alignment_span += len;
-                }
-                Feature::ReferenceSkip(_, len) => {
-                    alignment_span += len;
-                }
-                Feature::SoftClip(_, bases) => {
-                    alignment_span -= bases.len() as i32;
-                }
-                _ => {}
-            }
-        }
-
-        self.alignment_start() + alignment_span - 1
+        calculate_alignment_end(
+            self.alignment_start().map(i32::from).unwrap_or_default(),
+            self.read_length() as i32,
+            self.features(),
+        )
     }
 
     /// Returns the read group ID.
@@ -142,7 +127,9 @@ impl Record {
     }
 
     /// Returns the alignment start position of the next mate.
-    pub fn next_mate_alignment_start(&self) -> i32 {
+    ///
+    /// This value is 1-based.
+    pub fn next_mate_alignment_start(&self) -> Option<sam::record::Position> {
         self.next_mate_alignment_start
     }
 
@@ -163,10 +150,6 @@ impl Record {
         &self.tags
     }
 
-    pub(crate) fn add_tag(&mut self, tag: Tag) {
-        self.tags.push(tag);
-    }
-
     /// Returns the read bases.
     pub fn bases(&self) -> &[u8] {
         &self.bases
@@ -227,3 +210,50 @@ impl fmt::Debug for Record {
             .finish()
     }
 }
+
+fn calculate_alignment_span(read_length: i32, features: &[Feature]) -> i32 {
+    features
+        .iter()
+        .fold(read_length, |alignment_span, feature| match feature {
+            Feature::Insertion(_, bases) => alignment_span - bases.len() as i32,
+            Feature::InsertBase(_, _) => alignment_span - 1,
+            Feature::Deletion(_, len) => alignment_span + len,
+            Feature::ReferenceSkip(_, len) => alignment_span + len,
+            Feature::SoftClip(_, bases) => alignment_span - bases.len() as i32,
+            _ => alignment_span,
+        })
+}
+
+fn calculate_alignment_end(alignment_start: i32, read_length: i32, features: &[Feature]) -> i32 {
+    let alignment_span = calculate_alignment_span(read_length, features);
+    alignment_start + alignment_span - 1
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_calculate_alignment_span() {
+        let features = [];
+        assert_eq!(calculate_alignment_span(4, &features), 4);
+
+        let features = [Feature::HardClip(1, 4)];
+        assert_eq!(calculate_alignment_span(4, &features), 4);
+
+        let features = [
+            Feature::Insertion(1, vec![b'A', b'C']),
+            Feature::InsertBase(4, b'G'),
+            Feature::Deletion(6, 3),
+            Feature::ReferenceSkip(10, 5),
+            Feature::SoftClip(16, vec![b'A', b'C', b'G', b'T']),
+        ];
+        assert_eq!(calculate_alignment_span(20, &features), 21);
+    }
+
+    #[test]
+    fn test_calculate_alignment_end() {
+        let features = [];
+        assert_eq!(calculate_alignment_end(1, 4, &features), 4);
+    }
+}
