@@ -10,12 +10,12 @@ pub struct Builder {
     flags: Flags,
     reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
     read_length: usize,
-    alignment_start: i32,
+    alignment_start: Option<sam::record::Position>,
     read_group_id: ReadGroupId,
     read_name: Vec<u8>,
     next_mate_flags: NextMateFlags,
     next_fragment_reference_sequence_id: Option<bam::record::ReferenceSequenceId>,
-    next_mate_alignment_start: i32,
+    next_mate_alignment_start: Option<sam::record::Position>,
     template_size: i32,
     distance_to_next_fragment: i32,
     tags: Vec<Tag>,
@@ -60,8 +60,8 @@ impl Builder {
     }
 
     /// Sets the alignment start position.
-    pub fn set_alignment_start(mut self, alignment_start: i32) -> Self {
-        self.alignment_start = alignment_start;
+    pub fn set_alignment_start(mut self, alignment_start: sam::record::Position) -> Self {
+        self.alignment_start = Some(alignment_start);
         self
     }
 
@@ -93,8 +93,11 @@ impl Builder {
     }
 
     /// Sets the alignment start position of the next mate.
-    pub fn set_next_mate_alignment_start(mut self, next_mate_alignment_start: i32) -> Self {
-        self.next_mate_alignment_start = next_mate_alignment_start;
+    pub fn set_next_mate_alignment_start(
+        mut self,
+        next_mate_alignment_start: sam::record::Position,
+    ) -> Self {
+        self.next_mate_alignment_start = Some(next_mate_alignment_start);
         self
     }
 
@@ -197,12 +200,12 @@ impl Default for Builder {
             flags: Flags::default(),
             reference_sequence_id: None,
             read_length: 0,
-            alignment_start: 0,
+            alignment_start: None,
             read_group_id: ReadGroupId::default(),
             read_name: Vec::new(),
             next_mate_flags: NextMateFlags::default(),
             next_fragment_reference_sequence_id: None,
-            next_mate_alignment_start: 0,
+            next_mate_alignment_start: None,
             template_size: 0,
             distance_to_next_fragment: 0,
             tags: Vec::new(),
@@ -227,12 +230,12 @@ mod tests {
         assert_eq!(builder.flags, Flags::default());
         assert!(builder.reference_sequence_id.is_none());
         assert_eq!(builder.read_length, 0);
-        assert_eq!(builder.alignment_start, 0);
+        assert!(builder.alignment_start.is_none());
         assert_eq!(builder.read_group_id, ReadGroupId::default());
         assert!(builder.read_name.is_empty());
         assert_eq!(builder.next_mate_flags, NextMateFlags::default());
         assert!(builder.next_fragment_reference_sequence_id.is_none());
-        assert_eq!(builder.next_mate_alignment_start, 0);
+        assert!(builder.next_mate_alignment_start.is_none());
         assert_eq!(builder.template_size, 0);
         assert_eq!(builder.distance_to_next_fragment, 0);
         assert!(builder.tags.is_empty());
